SuperStrict

Rem
bbdoc: Database Driver - mSQL
about: A mSQL database driver for #bah.database.Database
End Rem
Module BaH.DBmSQL

ModuleInfo "Version: 1.01"
ModuleInfo "Author: Bruce A Henderson"
ModuleInfo "License: Free for non-commercial use"
ModuleInfo "License: Licensed mSQL required for commercial use."
ModuleInfo "Copyright: Wrapper - 2007 Bruce A Henderson"
ModuleInfo "Modserver: BRL"

ModuleInfo "History: 1.01"
ModuleInfo "History: Modified for Win32 compile."
ModuleInfo "History: 1.00 Initial Release (mSQL 3.8)"

Import BaH.Database


Import "common.bmx"


Type TDBmSQL Extends TDBConnection

	Field handle:Int

	Function Create:TDBConnection(dbname:String = Null, host:String = Null, ..
		port:Int = Null, user:String = Null, password:String = Null, server:String = Null, options:String = Null)
		
		Local this:TDBmSQL = New TDBmSQL
		
		this.init(dbname, host, port, user, password, server, options)
		
		If this._dbname Then
			this.open(user, password)
		End If
		
		Return this
		
	End Function

	Method close()
	
		If handle Then
		
			If msqlClose(handle) = -1 Then
				setError("Error closing connection", nativeErrorMessage(0), TDatabaseError.ERROR_CONNECTION, 0)
			End If
			
			handle = 0
			
		End If
		
		_isOpen = False
	End Method
	
	Method commit:Int()
		Return True
	End Method
	
	Method getTables:String[]()
		Local list:String[]
	
		If _isOpen Then
			Local query:TDatabaseQuery = TDatabaseQuery.Create(Self)
			
			query.resultSet = TmSQLResultSet.Create(Self)
		
			TmSQLResultSet(query.resultSet).listTables()
			
			Local tableList:TList = New TList
			
			For Local record:TQueryRecord = EachIn query
				tableList.addLast(record.getString(0))
			Next

			If tableList.count() > 0 Then
				list = New String[tableList.count()]
				Local i:Int = 0
				For Local s:String = EachIn tableList
					list[i] = s
					i:+ 1
				Next
			End If
			
		End If
		
		Return list
	End Method
	
	Method open:Int(user:String = Null, pass:String = Null)

		If _isOpen Then
			close()
		End If


		If user Then
			_user = user
		End If
		
		If pass Then
			_password = pass
		End If

		If _host And _host.length > 0 Then
			handle = msqlConnect(_host)
		Else
			handle = msqlConnect(Null)
		End If
		
		' open the database connection
		If handle <> -1 Then
		
			Local result:Int = msqlSelectDB(handle, _dbname)
			
			If result = -1 Then
				setError("Error selecting database", nativeErrorMessage(0), TDatabaseError.ERROR_CONNECTION, 0)
				Return False
			End If
		
		Else
			setError("Error connecting to database", nativeErrorMessage(0), TDatabaseError.ERROR_CONNECTION, 0)
			Return False
		End If

		_isOpen = True
		Return True
	End Method

	Method rollback:Int()
		Return True
	End Method
	
	Method startTransaction:Int()
		Return True
	End Method

	Method databaseHandle:Byte Ptr()
	End Method
	
	Method createResultSet:TQueryResultSet()
		Return TmSQLResultSet.Create(Self)
	End Method
	
	Method nativeErrorMessage:String(err:Int)
		Return String.fromCString(msqlGetErrMsg(Null))
	End Method
	
	Method hasPrepareSupport:Int()
		Return False
	End Method

	Method hasTransactionSupport:Int()
		Return False
	End Method

End Type


Type TmSQLResultSet Extends TQueryResultSet

	Field resultHandle:Byte Ptr
	Field _rowsAffected:Int

	Function Create:TQueryResultSet(db:TDBConnection, sql:String = Null)
		Local this:TmSQLResultSet = New TmSQLResultSet
		
		this.init(db, sql)
		this.rec = TQueryRecord.Create()
		
		Return this
	End Function

	Method Delete()
		If resultHandle Then
			msqlFreeResult(resultHandle)
			resultHandle = Null
		End If
	End Method

	Method executeQuery:Int(statement:String)

		If Not conn.isOpen() Then
			Return False
		End If
		
		If resultHandle Then
			msqlFreeResult(resultHandle)
			resultHandle = Null
		End If

		Local result:Int = msqlQuery(TDBmSQL(conn).handle, statement)
		
		If result = -1 Then
			conn.setError("Error executing query", conn.nativeErrorMessage(0), TDatabaseError.ERROR_STATEMENT, 0)
			Return False		
		End If
		
		_rowsAffected = result
		
		resultHandle = msqlStoreResult()
		
		If resultHandle Then
		
			Local fieldCount:Int = msqlFieldCount(resultHandle)
	
			initRecord(fieldCount)
	
			If fieldCount <> 0 Then
				
				For Local i:Int = 0 Until fieldCount
				
					Local _field:Byte Ptr = msqlFetchField(resultHandle)
	
					Local qf:TQueryField = TQueryField.Create(convertUTF8toISO8859(bmx_msql_field_name(_field)), dbTypeFromNative(Null, bmx_msql_field_type(_field), bmx_msql_field_flags(_field)))
					qf.length = bmx_msql_field_length(_field)
					qf.dflags = bmx_msql_field_flags(_field)
					qf.dtype = bmx_msql_field_type(_field)
					If qf.dflags & ~NOT_NULL_FLAG Then
						qf.nullable = True
					End If
					'qf.precision = bmx_msql_field_decimals(_field)
	
					rec.setField(i, qf)
				Next
				
			End If
		End If
		
		_isActive = True
		Return True
	End Method
	
	Method prepare:Int(statement:String)
		Return False ' no prepared statements !
	End Method
	
	Method execute:Int()
		Return False ' no prepared statements !
	End Method
	
	Method firstRow:Int()
		If index = SQL_BeforeFirstRow Then
			Return nextRow()
		End If
		
		Return False
	End Method
	
	Method nextRow:Int()

		If Not resultHandle Then
			Return False
		End If

		Local row:Byte Ptr Ptr = msqlFetchRow(resultHandle)
		
		If Not row
			Return False
		End If

		' now populate the values[] array with the fetched data !
		For Local i:Int = 0 Until rec.count()
		
			If values[i] Then
				values[i].clear()
			End If
			
			Local _field:TQueryField = rec.getField(i)

			If row[i] Then
			
				Local fieldLength:Int = bmx_msql_getFieldWidth(_field.dtype)
				If fieldLength < 0 Then
					fieldLength = _field.length
				End If
			
				Select rec.fields[i].fType
					Case DBTYPE_INT
						values[i] = New TDBInt
						values[i].setInt(String.fromBytes(row[i], fieldLength).toInt())
					Case DBTYPE_LONG
						values[i] = New TDBLong
						values[i].setLong(String.fromBytes(row[i], fieldLength).toLong())
					Case DBTYPE_FLOAT
						values[i] = New TDBFloat
						values[i].setFloat(String.fromBytes(row[i], fieldLength).toFloat())
					Case DBTYPE_DOUBLE
						values[i] = New TDBDouble
						values[i].setDouble(String.fromBytes(row[i], fieldLength).toDouble())
					Case DBTYPE_DATE
						' TODO
					Case DBTYPE_BLOB
						' TODO
					Default
						values[i] = New TDBString
						values[i].setString(convertUTF8toISO8859(row[i]))
				End Select
			
			End If
			
		Next

		index:+ 1
		
		Return True
	End Method
	
	Method lastInsertedId:Long()
	End Method
	
	Method rowsAffected:Int()
		Return _rowsAffected
	End Method

	Method dbTypeFromNative:Int(name:String, _type:Int = 0, _flags:Int = 0)

		Local dbType:Int
		
		Select _type
			Case INT_TYPE, UINT_TYPE, INT8_TYPE, INT16_TYPE, UINT8_TYPE, UINT16_TYPE
				dbType = DBTYPE_INT
			Case INT64_TYPE, UINT64_TYPE
				dbType = DBTYPE_LONG
			Case MONEY_TYPE
				dbType = DBTYPE_FLOAT
			Case REAL_TYPE
				dbType = DBTYPE_DOUBLE
			Default
				dbType = DBTYPE_STRING
		End Select
		
		Return dbType
	End Method

	Method initRecord(size:Int)

		rec.clear()

		If size > 0 Then		
			rec.init(size)
		End If
		
		resetValues(size)
	End Method

	Method listTables:Int()
	
		resultHandle = msqlListTables(TDBmSQL(conn).handle)
		
		If resultHandle Then
		
			Local fieldCount:Int = msqlFieldCount(resultHandle)
	
			initRecord(fieldCount)
	
			If fieldCount <> 0 Then
				
				For Local i:Int = 0 Until fieldCount
				
					Local _field:Byte Ptr = msqlFetchField(resultHandle)
	
					Local qf:TQueryField = TQueryField.Create(convertUTF8toISO8859(bmx_msql_field_name(_field)), dbTypeFromNative(Null, bmx_msql_field_type(_field), bmx_msql_field_flags(_field)))
					qf.length = bmx_msql_field_length(_field)
					qf.dflags = bmx_msql_field_flags(_field)
					qf.dtype = bmx_msql_field_type(_field)
					If qf.dflags & ~NOT_NULL_FLAG Then
						qf.nullable = True
					End If
	
					rec.setField(i, qf)
				Next
				
			End If
		End If
		
		_isActive = True
		Return True
	End Method
End Type


Type TmSQLDatabaseLoader Extends TDatabaseLoader

	Method New()
		_type = "MSQL"
	End Method

	Method LoadDatabase:TDBConnection( dbname:String = Null, host:String = Null, ..
		port:Int = Null, user:String = Null, password:String = Null, ..
		server:String = Null, options:String = Null )
	
		Return TDBmSQL.Create(dbName, host, port, user, password, server, options)
		
	End Method

End Type

AddDatabaseLoader New TmSQLDatabaseLoader

